/* -*- tab-width: 4 -*-
 *
 * Electric(tm) VLSI Design System
 *
 * File: ErrorToElectricReporter.java
 *
 * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
 *
 * Electric(tm) is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Electric(tm) is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Electric(tm); see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, Mass 02111-1307, USA.
 */
package com.sun.electric.api.minarea.errorLogging;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.util.LinkedList;
import java.util.List;

import com.sun.electric.api.minarea.MinAreaChecker;
import com.sun.electric.api.minarea.launcher.Launcher;
import java.util.Collections;
import java.util.Comparator;

/**
 * @author Felix Schmidt
 * 
 */
public class ErrorToElectricReporter implements MinAreaChecker.ErrorLogger {

	private String topCellName;

	public ErrorToElectricReporter(String topCellName) {
		this.topCellName = topCellName;
	}

	private class MinAreaViolation {

		private final long minArea;
		private final int x, y;

		public MinAreaViolation(long minArea, int x, int y) {
			this.minArea = minArea;
			this.x = x;
			this.y = y;
		}
	}

	private List<ErrorToElectricReporter.MinAreaViolation> violations = new LinkedList<ErrorToElectricReporter.MinAreaViolation>();

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.electric.api.minarea.MinAreaChecker.ErrorLogger#
	 * reportMinAreaViolation(long, int, int)
	 */
	public void reportMinAreaViolation(long area, int x, int y) {
		// TODO Auto-generated method stub
		violations.add(new MinAreaViolation(area, x, y));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.sun.electric.api.minarea.MinAreaChecker.ErrorLogger#printReports()
	 */
	public void printReports() {

		try {
			String pid = Launcher.getPID();
			String fileName = "errorLog.xml." + pid;
			BufferedReader br = new BufferedReader(
					new InputStreamReader(ErrorToElectricReporter.class.getResourceAsStream("loggerHeader.xml")));
			PrintStream pstream = new PrintStream(new File(fileName));

			String line = null;
			while ((line = br.readLine()) != null) {
				pstream.println(line);
			}

			pstream.println("<ErrorLogger errorSystem=\"MinArea DRC\">");
                        String cellName = this.topCellName;
                        if (cellName.indexOf('{') == -1 && cellName.indexOf('}') == -1)
                            cellName += "{lay}";
			pstream.println("\t<GroupLog message=\"" + cellName + "\">");

                        sortViolations();
			for (ErrorToElectricReporter.MinAreaViolation violation : violations) {
				pstream.println("\t\t<WarningLog message=\"Minimum Area Violation: actual = "
						+ String.valueOf(violation.minArea) + "\" cellName=\"" + cellName + "\">");
				pstream.println("\t\t\t<ERRORTYPEPOINT pt=\"(" + scaleCoord(violation.x) + "," + scaleCoord(violation.y)
						+ ")\" cellName=\"" + cellName + "\"/>");
				pstream.println("\t\t</WarningLog>");
			}

			pstream.println("\t</GroupLog>");
			pstream.println("</ErrorLogger>");

			br.close();
			pstream.flush();
			pstream.close();

			System.out.println("Output file written: " + fileName);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

    private static double scaleCoord(int coord) {
        return coord/100.;
    }
        
    private synchronized void sortViolations() {
        Collections.sort(violations, new Comparator<MinAreaViolation>() {

            public int compare(MinAreaViolation v1, MinAreaViolation v2) {
                if (v1.x > v2.x) {
                    return 1;
                }
                if (v1.x < v2.x) {
                    return -1;
                }
                if (v1.y > v2.y) {
                    return 1;
                }
                if (v1.y < v2.y) {
                    return -1;
                }
                return 0;
            }
        });
    }
}
